// 
//  Copyright (C) 1998,2007,2011,2016  Smithsonian Astrophysical Observatory
//
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along
//  with this program; if not, write to the Free Software Foundation, Inc.,
//  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
//


#ifndef DS_ALLDATATYPES_H
#define DS_ALLDATATYPES_H

#include <iostream>
#include <string>
using std::string;
using std::cout;
using std::cin;
using std::endl;
#include <map>
#include <deque>
using std::map;
using std::deque;


#ifndef LIBERR_H
#include <liberr.h>
#endif

#ifndef DS_ANCILLARY_OBJ_H
#include "dsAncillaryObj.hh"
#endif

#define dsFront 		0
#define dsBack  		1

/* enumeration for detailing data types.  It is important that dsText be the 
   first of the types that is not convertable.  This includes Bit Arrays, Bytes,
   and perhaps booleans.  If an unsigned short is added, say, it should be
   added to the list in the appropriate place, and all the numbers reassigned. */
enum dsDataTypes {dsBool = 1, dsUShort = 2, dsShort = 3, dsUChar = 4, dsULong = 5, dsInt = 6, dsLong = 7, dsFloat = 8, dsDouble = 9, dsText = 10, dsBit = 11, dsUnknown = 12};

/* structure to hold one of a variety of types of data */
typedef union dsdataunion {
  unsigned short ushort_val;
  short short_val;
  unsigned char uchar_val;
  int int_val;
  unsigned long ulong_val;
  long long_val;
  float float_val;
  double double_val;
  string *string_val_p;
  dsBitArray *bit_val_p;
  unsigned short *ushort_val_a;
  short *short_val_a;
  unsigned char *uchar_val_a;
  int *int_val_a;
  unsigned long *ulong_val_a;
  long *long_val_a;
  float *float_val_a;
  double *double_val_a;
  string **string_val_a;
  dsBitArray **bit_val_a;
} dsDataUnion;

/* object to hold one of a variety of types of data */
class dsAllDataTypes
{
public:
  
  /* constructors */
  dsAllDataTypes(void);
  dsAllDataTypes(unsigned short val);
  dsAllDataTypes(short val);
  dsAllDataTypes(int val);
  dsAllDataTypes(unsigned char val);
  dsAllDataTypes(unsigned long val);
  dsAllDataTypes(long val);
  dsAllDataTypes(float val);
  dsAllDataTypes(double val);
  dsAllDataTypes(string val);
  dsAllDataTypes(unsigned char *new_data_p,
		 long new_num_bits,
		 long size_array);
  dsAllDataTypes(unsigned short *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(short *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(int *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(unsigned char *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(unsigned long *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(long *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(float *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(double *vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(string **vals_p,
		 dsDimensions &new_dims);
  dsAllDataTypes(unsigned char **new_data_p,
		 long new_num_bits,
		 long size_array,
		 dsDimensions &new_dims);

  dsAllDataTypes(dsDataTypes type_e,
		 long num_bits = 0);

  dsAllDataTypes(dsDataTypes type_e,
		 const dsDimensions &dims,  
		 long num_bits = 0);


  /* destructor */
  ~dsAllDataTypes(void){HelpDestructor();};

  /* copy constructor */
  dsAllDataTypes(const dsAllDataTypes &copy);

  /* assignment operators */
  dsAllDataTypes &operator = (const dsAllDataTypes &right);
  unsigned short operator = (unsigned short right) {HelpDestructor(); 
  						    data.ushort_val = right; 
						    axes = dsDimensions();
						    recent_type_e = dsUShort;
						    abstract_data_p = &data.ushort_val;
						    return right;};
  short operator = (short right) {HelpDestructor(); data.short_val = right; 
  				  axes = dsDimensions(); recent_type_e = dsShort;
				  abstract_data_p = &data.short_val; return right;};
  int operator = (int right) {HelpDestructor(); data.int_val = right; 
  			      axes = dsDimensions(); recent_type_e = dsInt; 
			      abstract_data_p = &data.int_val; return right;};
  unsigned char operator = (unsigned char right) {HelpDestructor();
  						  data.uchar_val = right;
						  axes = dsDimensions();
						  recent_type_e = dsUChar;
						  abstract_data_p = &data.uchar_val;
						  return right;}
  unsigned long operator = (unsigned long right) {HelpDestructor(); 
  						  data.ulong_val = right; 
						  axes = dsDimensions();
						  recent_type_e = dsULong;
						  abstract_data_p = &data.ulong_val;
						  return right;};
  long operator = (long right) {HelpDestructor(); data.long_val = right; 
  				axes = dsDimensions(); recent_type_e = dsLong;
				abstract_data_p = &data.long_val; return right;};
  float operator = (float right) {HelpDestructor(); data.float_val = right; 
  				  axes = dsDimensions(); recent_type_e = dsFloat;
				  abstract_data_p = &data.float_val; return right;};
  double operator = (double right) {HelpDestructor(); data.double_val = right; 
  				    axes = dsDimensions(); recent_type_e = dsDouble;
				    abstract_data_p = &data.double_val; 
				    return right;};
  string &operator = (string &right) {HelpDestructor(); 
  				      data.string_val_p = new string(right);
				      axes = dsDimensions(); recent_type_e = dsText;
		 		      abstract_data_p = &data.string_val_p;
				      return right;};
  
  dsBitArray operator = (dsBitArray right) {HelpDestructor();
  data.bit_val_p = new dsBitArray(right);
  axes = dsDimensions(); 
  recent_type_e = dsBit;
  abstract_data_p = &data.bit_val_p;
  return right;};
  
  /* conversion (cast) operators */
  operator unsigned short(void);
  operator short(void);
  operator int(void);
  operator unsigned char(void);
  operator unsigned long(void);
  operator long(void);
  operator float(void);
  operator double(void);
  operator string(void);
  operator dsBitArray(void);

  operator unsigned short*(void);
  operator short*(void);
  operator int*(void);
  operator unsigned char*(void);
  operator unsigned long*(void);
  operator long*(void);
  operator float*(void);
  operator double*(void);

  /* basic mathematic operators - WARNING, these will do nothing if either
     recent_type_e is dsText.  If the types are not the same, it will attempt
     conversion. */
  dsAllDataTypes operator + (const dsAllDataTypes &right);
  dsAllDataTypes operator - (const dsAllDataTypes &right);
  dsAllDataTypes operator / (const dsAllDataTypes &right);
  dsAllDataTypes operator * ( const dsAllDataTypes &right);
  dsAllDataTypes operator % (const dsAllDataTypes &right);

  /* convenience mathematic operators.  Eventually, all should be 
     done */
  dsAllDataTypes operator += (const dsAllDataTypes &right);
  //  dsAllDataTypes operator -= (dsAllDataTypes &right);
  //  dsAllDataTypes operator /= (dsAllDataTypes &right);
  //  dsAllDataTypes operator *= (dsAllDataTypes &right);


  /* boolean operators - WARNING, these will always return false if the
     recent_type_e is not the same and one type is dsText.  If both
     types are numbers, then it will perform an internal conversion.  */
  bool operator ! (void) const;
  bool operator != (const dsAllDataTypes &right) const;
  bool operator == (const dsAllDataTypes &right) const;
  bool operator < (const dsAllDataTypes &right) const;
  bool operator > (const dsAllDataTypes &right) const;
  bool operator <= (const dsAllDataTypes &right) const;
  bool operator >= (const dsAllDataTypes &right) const;
  
  /* accessors */
  unsigned short GetUShortVal(unsigned long index = 0);
  short GetShortVal(unsigned long index = 0);
  int GetIntVal(unsigned long index = 0);
  int GetBoolVal(unsigned long index = 0);
  unsigned char GetUCharVal(unsigned long index = 0);
  unsigned long GetULongVal(unsigned long index = 0);
  long GetLongVal(unsigned long index = 0);
  float GetFloatVal(unsigned long index = 0);
  double GetDoubleVal(unsigned long index = 0);
  string GetStringVal(unsigned long index = 0);
  dsBitArray GetBitVal(unsigned long index = 0);
  dsAllDataTypes GetNthVal(unsigned long index);
  long GetCellLength(void){return cell_length;};

  dsDataTypes GetType(void) {return recent_type_e;};
  dsDimensions GetDimensions(void) {return axes;};

  /* modifiers */
  dsErrBool SetUShortVal(unsigned short val, unsigned long index = 0);
  dsErrBool SetShortVal(short val, unsigned long index = 0);
  dsErrBool SetIntVal(int val, unsigned long index = 0);
  dsErrBool SetBoolVal(int val, unsigned long index = 0);
  dsErrBool SetUCharVal(unsigned char val, unsigned long index = 0);
  dsErrBool SetULongVal(unsigned long val, unsigned long index = 0);
  dsErrBool SetLongVal(long val, unsigned long index = 0);
  dsErrBool SetFloatVal(float val, unsigned long index = 0);
  dsErrBool SetDoubleVal(double val, unsigned long index = 0);
  dsErrBool SetStringVal(string val, unsigned long index = 0);
  dsErrBool SetBitVal(dsBitArray val, unsigned long index = 0);
  dsErrBool SetNthVal(dsAllDataTypes val, unsigned long index = 0);

  void SetType(dsDataTypes new_type_e,
	       long num_bits = 0);
  void SetDimensions(dsDimensions new_dims);
  dsErrBool Convert2Type(dsDataTypes new_type_e);
  
  /* miscellaneous */
  void Print(void);

protected:
  void HelpConstructor(const dsAllDataTypes &other);
  inline void HelpDestructor(void);
  void HelpPrint(void *start_p, 
		 long num_dimensions,
		 vector<long>::iterator dimensions);

  short setup_for_operators(const dsAllDataTypes &other, 
			   void *&larger_p,
			   void *&smaller_p,
			   unsigned long &larger_limit,
			   unsigned long &smaller_limit,
			   short &larger_incr,
			   short &smaller_incr,
			   dsDimensions &larger_dimensions) const;
			   

  dsDataTypes recent_type_e;
  dsDataUnion data;
  dsDimensions axes;
  long cell_length; // nominal length of bit and text values (array size)
  void *abstract_data_p;
};



/* function to help in destruction of the union - we need to ensure that 
   the string and bit objects, and arrays, get destroyed properly */
void dsAllDataTypes::HelpDestructor(void)
{

  if(axes.GetNumDimensions() > 0)
    {
	  
      switch(recent_type_e)
	{
	case dsShort:
	  {
	    delete[] data.short_val_a;
	    break;
	  }
	case dsUShort:
	  {
	    delete[] data.ushort_val_a;
	    break;
	  }
	case dsInt:
	  {
	    delete[] data.int_val_a;
	    break;
	  }
	case dsUChar:
	  {
	    delete[] data.uchar_val_a;
	    break;
	  }
	case dsLong:
	  {
	    delete[] data.long_val_a;
	    break;
	  }
	case dsULong:
	  {
	    delete[] data.ulong_val_a;
	    break;
	  }
	case dsFloat:
	  {
	    delete[] data.float_val_a;
	    break;
	  }
	case dsDouble:
	  {
	    delete[] data.double_val_a;
	    break;
	  }
	case dsBool:
	  {
	    delete[] data.int_val_a;
	    break;
	  }
	case dsText:
	  {
	    unsigned long ctr = 0;
	    for(ctr=0; ctr < axes.GetArraySize(); ctr++)
	      {
		delete data.string_val_a[ctr];
	      }
	    delete[] data.string_val_a;
	    break;
	  }
	case dsBit:
	  {
	    unsigned long ctr = 0;
	    for(ctr=0; ctr < axes.GetArraySize(); ctr++)
	      {
		delete data.bit_val_a[ctr];
	      }
	    delete[] data.bit_val_a;
	    break;
	  }
	default:
	  {
	    break;
	  }
	}
    }
  else
    {
      if(recent_type_e == dsText)
	{
	  delete data.string_val_p;
	  data.string_val_p = NULL;
	}
      else if(recent_type_e == dsBit)
	{
	  delete data.bit_val_p;
	  data.bit_val_p = NULL;
	}
    }

  axes = dsDimensions();
  abstract_data_p = NULL;
  cell_length = 0;
}







#endif
