#!/usr/bin/perl
# 
#  Copyright (C) 2004-2005,2007  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
unshift(@INC, $TEXEC_DIR);
require 'test_utils.pl';
$RM = find_inpath( "rm", $ENV{'PATH'}, ':' );
$MKDIR = find_inpath( "mkdir", $ENV{'PATH'}, ':' );
$PUNLEARN = find_inpath( 'punlearn', $ENV{'PATH'}, ':' );
$DMCOPY = find_inpath( 'dmcopy', $ENV{'PATH'}, ':' );

use strict;
use vars '$RM', '$MKDIR', '$PUNLEARN', '$ASCDS_INSTALL', '$DMCOPY';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '002';
my $semantic = 'dmcopy';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;

#
# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";

# Process Date
my $date = `date +%d%b%Y_%H:%M:%S`;
chop($date);
my ($junk,$start_hours,$start_min,$start_sec ) = $date =~ /(.*_)(\d*):(\d*):(\d*)/;

# set platform
my $platform = `uname`;
chop $platform;


##########################################################################
# Initialize some default values
my $USER    = $ENV{'USER'};
my $UPARM   = $ENV{'ASCDS_USER_PARAM'};
my $version = $ENV{'ASCDS_VERSION'};
my $home    = $ENV{'HOME'};
my $thisdir = $ENV{'PWD'};
my $testin  = "$ENV{'TESTIN'}/datamodel";
my $testout = "$ENV{'TESTOUT'}/datamodel";
my $testlog = "$ENV{'TESTLOG'}/datamodel";
my $testsav;
if ( -d "$ENV{'TESTSAV'}/datamodel" )
{
   $testsav = "$ENV{'TESTSAV'}/datamodel";
}
else
{
   $testsav = "$ASCDS_INSTALL/test/baseline/datamodel";
}
my $testrun = $ENV{'TESTRUN'};
my $printfile = *STDOUT;
my $caldb   = $ENV{'CALDB'};
my $filterfile = $testin;


#
#------------------------------------------------------------
# Load Arguments
my %inargs;
%inargs = &parse_args(\@ARGV);

#
# Process Arguments

# ***********
# -h = Help
# ***********
if ( $inargs{"-h"} )
{
   print "Usage: \n";
   print "   $script_name [ -iosfph ] \n";
   print "      -h   Help ( this text ) \n";
   print "\n";
   print "      -o   Specify test output directory \n";
   print "\n";
   print "      -i   Specify test input directory \n";
   print "\n";
   print "      -s   Specify test saved output directory \n";
   print "\n";
   print "      -f   Specify log file \n";
   print "\n";
   print "      -p   Specify printed output file \n";
   print "\n";
   print "      -k   Specify kernel, e.g., FITS or TEXT \n";
   print "\n\n";
   exit 0;
}

# ***********
# -i = TESTIN directory
# ***********
if ( $inargs{'-i'} )
{
   $testin = $inargs{'-i'};
}
# ***********
# -o = TESTOUT directory
# ***********
if ( $inargs{'-o'} )
{
   $testout = $inargs{'-o'};
}
# ***********
# -s = TESTSAV directory
# ***********
if ( $inargs{'-s'} )
{
   $testsav = $inargs{'-s'};
}
# make sure testing directories exist
unless ( -d $testlog ) { system( "$MKDIR -p $testlog" ); }
unless ( -d $testout ) { system( "$MKDIR -p $testout" ); }
unless ( -d $testin )
{
   print $printfile "ERROR: can't find input directory : $testin\n";
   exit 1;
}
unless ( -d $testsav )
{
   print $printfile "ERROR: can't find saved output directory : $testsav\n";
   exit 1;
}

# Clean up old log files
my $comstr = sprintf( "$RM -f $testlog/dt%.3d.log.*", $id );
system( $comstr );

# Open log files

# ***********
# -p = verbose output file
# ***********
if ( $inargs{'-p'} )
{
   unless ( open PRINT_OUTPUT, ">>$inargs{'-p'}" )
   {
      print STDOUT "ERROR: Failed to open verbose output file : $inargs{'-p'}\n";
      exit 1;
   }
   $printfile = *PRINT_OUTPUT;
   # turn off buffering to print file
   select($printfile); $| = 1;
}

my $logfile;
my $logfname = "$testlog/dt${id}.log.$date";
# ***********
# -f = LOG file
# ***********
if ( $inargs{'-f'} )
{
   $logfname = $inargs{'-f'};
}

if ( open LOG_OUTPUT, ">>$logfname" )
{
   $logfile = *LOG_OUTPUT;
}
else
{
   print $printfile "ERROR: Failed to open log file : $logfname\n";
   exit 1;
}
# turn off buffering for the logfile
select($logfile); $| = 1; select($printfile);

# for specified test kernel
my $testKernel = $ARGV[0];

# ***********
# -k = Kernel
# ***********
# anything to do here?
if ( $inargs{'-k'} )
{
   $testKernel = $inargs{'-k'};
}

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################

my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = ( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22
);
my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 "1" => { "image" => [ 1 ]},
 "2" => { "table" => [ 1, 2 ]},
 "3" => { "list" => [ ] },
 "4" => { "table" => [ 1, 2, 3 ]},
 "5" => { "image" => [ 1 ],
	  "table" => [ 2 ]},
 "6" => { "table" => [ 1, 2 ]},
 "7" => { "table" => [ 1, 2 ]},
 "8" => { "image" => [ 1 ],
	  "table" => [ 2, 3, 4, 5, 6 ]},
 "9" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "10" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "11" => { "table" => [ 1, 2, 3 ] },
 "12" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ] },
 "13" => { "table" => [ 2 ] },
 "14" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "15" => { "table" => [ 1, 2, 3, 4, 5, 6, 7 ]},
 "16" => { "table" => [ 2 ],
           "image" => [ 1 ] },
 "17" => { "list" => [ ] },
 "18" => { "image" => [ 1 ]},
 "19" => { "table" => [ 1, 2, 3, 4 ] },
 "20" => { "table" => [1] },
 "21" => { "list" => [ ] },
 "22" => { "list" => [ ] }
);
		     
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );


dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my @faillist = ( );
my $time_user_total = 0;
my $time_sys_total = 0;
my $rc;
my $err_msg = "";

my %exten = ("FITS" => "fits",
             "TEXT" => "dtf" );
my %copyOpts = ("TEXT" => "text/dtf");
my %links = ("8" => "indt002m007",
             "9" => "indt002m007",
             "10" => "indt002m007");
my $kernelName = "";
my $line = "";
my $pid;
my @kernels;
my $test_string;
my $in_ext = "fits";
my $out_ext = "fits";
my $source_ext = "fits";
#########################################################################

# start the child process to get the kernel list
$pid = open( INHANDLE, "getkernellist |");

# get the output of the child process
$line = <INHANDLE>;

# break the output line into individual datum and put
# into list of kernel names
@kernels = split(/\s+/, $line);

close(INHANDLE);

# Control looping over each installed kernel
# by checking for kernel argument.
# If present, perform tests only the specified kernel
# If absent, perform tests on each installed kernel

foreach $kernelName (@kernels)
{
  my $errFlag = 1;

  if (($testKernel ne "") && ($kernelName ne $testKernel))
  {
    next;
  }

  # Loop over all tests
  foreach my $testid (@testlist)
  {
    # Init per-test error flag
    my $mismatch = 0;
    my $time_user;
    my $time_sys;
    $err_msg = "";
    my $skipDelete = 0;
    my $linkname = "";

    # change extension based on kernel
    $in_ext = $exten{$kernelName};

    # delete old outputs
    my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
    system("$RM -f $testout/${basename}*");

    my $block_typesref = $test_blocks{$testid};
    my %block_types = %$block_typesref;

    my $infile = "$testin/in${basename}.$in_ext";

    # if testing any kernel other than FITS, the FITS input files need to
    # be copied into the present kernel's format
    if ( $kernelName ne "FITS" )
    {
      my $sourceFile = "$testin/in${basename}.$source_ext";
      my $copyFile   = "$testout/in${basename}.$in_ext";
      my $options    = "opt=all";
      my $kernelOpts = $copyOpts{"$kernelName"};


      if (!  -e "$infile" )
      {
        my $linkbase = "";
        my $teststring = "$DMCOPY $sourceFile $copyFile\"[opt kernel=$kernelOpts]\" $options";

        # does a previous copy file exist?
        if ($testid == 8 || $testid == 9 || $testid == 10 )
        {
          $linkbase = $links{"$testid"};
          $linkname = "$testout/${linkbase}.$in_ext";
        }

        # if not, make one
        if ( ! -e $copyFile )
        {
          if ( $linkname ne "" )
          {
            if ( -e $linkname )
            {
              # create link
              $teststring = "ln -s $linkname $copyFile";
            }
          }

          # copy fits file to text/dtf file
          ($rc, $time_user, $time_sys) = system($teststring);
        }

        $infile = $copyFile;

         # Tests 7, 8, 9 and 10 use the same input file.
         # Don't delete generated input file yet
         if ( $testid == 7 )
         {
           $skipDelete = 1;
         }
      }
      else
      {
        # preserve inputs
        $skipDelete = 1;
      }
    }

    # PROGRAMMER - put output filename here
    my $outfile = "$testout/${basename}.$out_ext";
    # PROGRAMMER - put saved output filename here
    my $savfile = "$testsav/${basename}.$out_ext";
    # PROGRAMMER - put name of diff file here, if needed
    my $test_diff = "$testout/${basename}.diff";
    # PROGRAMMER - put test string here
    my $test_string = "$testout/${basename}.diff";


    if ( $testid == 1 )
    {
      $test_string = "$DMCOPY infile=$infile outfile=$outfile";
    }
    elsif ( $testid == 2 )
    {
      $test_string = "$DMCOPY infile=$infile outfile=$outfile" ;
    }
    elsif ( $testid == 3 )
    {
      $test_string = "$DMCOPY infile=$infile outfile=$outfile";
    }
    elsif ( $testid == 4 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[stdevt\]\" outfile=$outfile";
    }
    elsif ( $testid == 5 )
    {
      $test_string = "$DMCOPY infile=\"$infile\[stdevt][bin x=32,y=32\]\" outfile=$outfile";
    }
    elsif ( $testid == 6 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[(x,y)=circle(4183,5870,200)*!annulus(4222,5970,100,1000)]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 7 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[ccd_id=5,time=58153664.094645:58160019.294651]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 8 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[bin x=3900:4300:2,y=3900:4300:2]' . "\" outfile=$outfile option=\"\"";
    }
    elsif ( $testid == 9 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[filter sky=region(' . "$testin/in${basename}.reg" . ')]' . "\" outfile=$outfile option=\"\"";
    }
    elsif ( $testid == 10 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[exclude sky=circle(4057.5,4161.5,26)]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 11 )
    {
      $test_string = "$DMCOPY infile=\"$infile" . '[status=0,grade=0,2,3,4,6]' . "\" outfile=$outfile";
    }
    elsif ( $testid == 12 )
    {
      $test_string = "$DMCOPY infile=\"$infile\" outfile=$outfile";
    }
    elsif ( $testid == 13 )
    {
     # check for Bug 9650
      $test_string = "$DMCOPY infile=\"$infile\" outfile=$outfile";
    }
    elsif ( $testid == 14 )
    {
     # test 11 from dmcopy.t
      $test_string = "$DMCOPY infile=$infile'[exclude pha=1:50][opt -ssupdate][newname]' outfile=$outfile";
    }
    elsif ( $testid == 15 )
    {
     # test 12 from dmcopy.t
      $test_string = "$DMCOPY infile=$infile'[exclude pha=1:50][newname]' outfile=$outfile";
    }
    elsif ( $testid == 16 )
    {
     # test 14 from dmcopy.t
      $test_string = "$DMCOPY infile=$infile'[stdevt][pha=100:300][bin x=32, y=32]' outfile=$outfile";
    }
    elsif ( $testid == 17 )
    {
     # test 15 from dmcopy.t
      $filterfile = "$testin/in${basename}.reg";
      $test_string = "$DMCOPY infile=$infile\'[(x,y)=region($filterfile)]\' outfile=$outfile";
    }
    elsif ( $testid == 18 )
    {
     # test 18 from dmcopy.t - check for Bug 8130 image rebinning
      $test_string = "$DMCOPY infile=$infile'[bin x=3826.5:4966.5:4.0,y=3516.5:4656.5:4.0]' outfile=$outfile";
    }
    elsif ( $testid == 19 )
    {
     # test 19 from dmcopy.t - check for Bug 8289 exclude + file subspace filter
      $filterfile = "${testin}/in${basename}_filter.fits[ccd_id=3]";
      $test_string = "$DMCOPY infile=$infile\'[exclude \@${filterfile}]\' outfile=$outfile";
    }
    elsif ( $testid == 20 )
    {
     # test 20 from dmcopy.t - check for Bug 9698 - losing filer info
     # if filter file and text used
      $filterfile = "${testin}/in${basename}_filter.txt,time=782478700,78250000";
      $test_string = "$DMCOPY infile=$infile\'[\@${filterfile}]\' outfile=$outfile";
    }
    elsif ( $testid == 21 )
    {
      # Test for Bug13670 - filtering an input file with no physical transform defined.
      # The resulting file should have the correct transform for the filter, as opposed
      # to a unit transform.
      $filterfile = "$testin/in${basename}.reg";
      $test_string = "$DMCOPY infile=$infile\'[(x,y)=rect(50,100,100,150)]\' outfile=$outfile";
    }
    elsif ( $testid == 22 )
    {
      # Test for Bug14267 - wherein the region filter applied to the file doesn't have any points in 
      # it because of non-overlapping shapes. The end result should be a file with no data in it (0x0 image or 0 row table).
      $test_string = "$DMCOPY infile=$infile\'[(#1,#2)=rect(10,10,20,20)&rect(30,30,40,40)]\' outfile=$outfile";
    }


    # run the tool
    $test_string .= " >> $logfname 2>&1";
    dt_print_test_header( $testid, $test_string, $logfile, $printfile, $kernelName );
    close $logfile;
    ( $rc, $time_user, $time_sys ) = dt_run_test( $test_string );
    open $logfile, ">>$logfname";
    select($logfile); $| = 1; select($printfile);
    unless ( $rc == 0 )
    {
      $err_msg = "ERROR: $toolname failed to run\n";
    }
 
    if ( $block_types{"table"} )
    {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
        # do not compare null primary blocks for ascii kernel
        if ($block != 1 || ($kernelName ne "TEXT") )
        {
          $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
        }
      }
    }

    if ( $block_types{"image"} )
    {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
        $err_msg .= dmimgcalc_test( $outfile, $savfile, $block,"$testin/tolerance", $logfname, $test_diff );
      }
    }

    if ( $block_types{"list"} )
    {
      $err_msg .= dmlist_test( $outfile, "${outfile}_filt", $savfile, "$testout/${basename}_savfilt", \@tolkeys, $logfname, $test_diff, "" );
    }

    unless ( $err_msg eq "" )
    {
      # Put the kernel name in front of the list of failed tests
      if ( $errFlag == 1 )
      {
        push (@faillist, $kernelName);
        $errFlag = 0;
      }
      push( @faillist, $testid );
      $mismatch = 1;
    }

    $time_user_total += $time_user;
    $time_sys_total += $time_sys;
    dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );

    # delete any input files created just for this kernel
    if ( $kernelName ne "FITS" )
    {
      if ($skipDelete == 0 )
      {
        system("$RM -f $infile");
      }
      if ( $testid == 10 )
      {
        system("$RM -f $linkname");
      }        
    }
  }
}

my $retval;
if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
