#!/usr/bin/perl
# 
#  Copyright (C) 2004-2007,2017  Smithsonian Astrophysical Observatory
#
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License along
#  with this program; if not, write to the Free Software Foundation, Inc.,
#  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#

@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( ! -e 'test_utils.pl' )
{
   if ( -d "./datamodel" )
   {
      $TEXEC_DIR = "./datamodel";
   }
   elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
   {
      $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
   }
}
unshift(@INC, $TEXEC_DIR);
require 'test_utils.pl';
$RM = find_inpath( "rm", $ENV{'PATH'}, ':' );
$MKDIR = find_inpath( "mkdir", $ENV{'PATH'}, ':' );
$PUNLEARN = find_inpath( 'punlearn', $ENV{'PATH'}, ':' );
$DMCOPY = find_inpath( 'dmcopy', $ENV{'PATH'}, ':' );

use strict;
use vars '$RM', '$MKDIR', '$PUNLEARN', '$ASCDS_INSTALL', '$DMCOPY';

##########################################################################
# PROGRAMMER - SET THESE VARIABLES!
my $id = '023';
my $semantic = 'asciiCopyTo';
my $script_name = 'dt${id}${semantic}.pl';

# Turn off buffering
select(STDOUT); $| = 1;
#
# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";
$ENV{'LC_ALL'} = "C";

# Process Date
my $date = `date +%d%b%Y_%H:%M:%S`;
chop($date);
my ($junk,$start_hours,$start_min,$start_sec ) = $date =~ /(.*_)(\d*):(\d*):(\d*)/;

# set platform
my $platform = `uname`;
chop $platform;


##########################################################################
# Initialize some default values
my $USER    = $ENV{'USER'};
my $UPARM   = $ENV{'ASCDS_USER_PARAM'};
my $version = $ENV{'ASCDS_VERSION'};
my $home    = $ENV{'HOME'};
my $thisdir = $ENV{'PWD'};
my $testin  = "$ENV{'TESTIN'}/datamodel";
my $testout = "$ENV{'TESTOUT'}/datamodel";
my $testlog = "$ENV{'TESTLOG'}/datamodel";

my $testsav;
if ( -d "$ENV{'TESTSAV'}/datamodel" )
{
   $testsav = "$ENV{'TESTSAV'}/datamodel";
}
else
{
   $testsav = "$ASCDS_INSTALL/test/baseline/datamodel";
}
my $testrun = $ENV{'TESTRUN'};
my $printfile = *STDOUT;
my $caldb   = $ENV{'CALDB'};



#
#------------------------------------------------------------
# Load Arguments
my %inargs;
%inargs = &parse_args(\@ARGV);

#
# Process Arguments

# ***********
# -h = Help
# ***********
if ( $inargs{"-h"} )
{
   print "Usage: \n";
   print "   $script_name [ -iosfph ] \n";
   print "      -h   Help ( this text ) \n";
   print "\n";
   print "      -o   Specify test output directory \n";
   print "\n";
   print "      -i   Specify test input directory \n";
   print "\n";
   print "      -s   Specify test saved output directory \n";
   print "\n";
   print "      -f   Specify log file \n";
   print "\n";
   print "      -p   Specify printed output file \n";
   print "\n\n";
   exit 0;
}

# ***********
# -i = TESTIN directory
# ***********
if ( $inargs{'-i'} )
{
   $testin = $inargs{'-i'};
}
# ***********
# -o = TESTOUT directory
# ***********
if ( $inargs{'-o'} )
{
   $testout = $inargs{'-o'};
}
# ***********
# -s = TESTSAV directory
# ***********
if ( $inargs{'-s'} )
{
   $testsav = $inargs{'-s'};
}
# make sure testing directories exist
unless ( -d $testlog ) { system( "$MKDIR -p $testlog" ); }
unless ( -d $testout ) { system( "$MKDIR -p $testout" ); }
unless ( -d $testin )
{
   print $printfile "ERROR: can't find input directory : $testin\n";
   exit 1;
}
unless ( -d $testsav )
{
   print $printfile "ERROR: can't find saved output directory : $testsav\n";
   exit 1;
}

# Clean up old log files
my $comstr = sprintf( "$RM -f $testlog/dt%.3d.log.*", $id );
system( $comstr );

# Open log files

# ***********
# -p = verbose output file
# ***********
if ( $inargs{'-p'} )
{
   unless ( open PRINT_OUTPUT, ">>$inargs{'-p'}" )
   {
      print STDERR "ERROR: Failed to open verbose output file : $inargs{'-p'}\n";
      exit 1;
   }
   $printfile = *PRINT_OUTPUT;
   # turn off buffering to print file
   select($printfile); $| = 1;
}

my $logfile;
my $logfname = "$testlog/dt${id}.log.$date";
# ***********
# -f = LOG file
# ***********
if ( $inargs{'-f'} )
{
   $logfname = $inargs{'-f'};
}
if ( open LOG_OUTPUT, ">>$logfname" )
{
   $logfile = *LOG_OUTPUT;
}
else
{
   print $printfile "ERROR: Failed to open log file : $logfname\n";
   exit 1;
}
# turn off buffering for the logfile
select($logfile); $| = 1;
select($printfile);

# Parse the tolerance file
# This is only necessary if this test uses the dmlist_test.
my $tolfile = sprintf("$testin/tol%.3d", $id);
my $tolkeysref = parse_tolfile( $tolfile );
my @tolkeys = @$tolkeysref;

######################### BEGIN TEST CODE ###################################

my $toolname = "dmcopy"; # PROGRAMMER - put tool name here
my @testlist = 
( 
# PROGRAMMER - List the test ids here
1,
2,
3,
4,
5,
6,
7,
8,
9,
10,
11,
12,
13,
14,
15,
16,
17,
18,
19,
20,
21,
22,
23,
24,
25,
26,
27,
28,
29,
30,
31,
32,
33,
34,
35,
36,
37
);
my %test_blocks = 
( 
# PROGRAMMER - put lists of blocks and block types for each test
# eg:
# "1" => { "image" => [ 1, 3 ],
#          "table" => [ 2, 4 ] },
# "2" => { "list" => [ ] },...
 1 => { "text" => [] },
 2 => { "text" => [] },
 3 => { "text" => [] },
 4 => { "text" => [] },
 5 => { "table" => [ 1, 2 ] },
 6 => { "text" => [] },
 7 => { "text" => [] },
 8 => { "text" => [] },
 9 => { "text" => [] },
 10 => { "table" => [ 1, 2 ] },
 11 => { "text" => [] },
 12 => { "text" => [] },
 13 => { "text" => [] },
 14 => { "text" => [] },
 15 => { "table" => [ 1, 2 ] },
 16 => { "text" => [] },
 17 => { "text" => [] },
 18 => { "text" => [] },
 19 => { "text" => [] },
 20 => { "table" => [ 1 ] },
 21 => { "text" => [] },
 22 => { "text" => [] },
 23 => { "text" => [] },
 24 => { "text" => [] },
 25 => { "table" => [ 1 ] },
 26 => { "text" => [] },
 27 => { "text2" => [] },
 28 => { "text2" => [] },
 29 => { "text2" => [] },
 30 => { "text2" => [] },
 31 => { "text2" => [] },
 32 => { "text" => [] },
 33 => { "text" => [] },
 34 => { "text" => [] },
 35 => { "text" => [] },
 36 => { "table" => [2] },
 37 => { "text2" => [] }
);
my %test_words = 
(
# PROGRAMMER - put list of words to replace in a text output file
#              This is only necessary if using the diff test for text files
#  ** Standard **
$version => "ASCDS_VERSION",
'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d' => "TEST_DATE",
 'HISTORY.*\n' => "",
 'HISTNUM.*' => "HISTNUM",
 'NaN' => "NAN",
 'nan\t' => "NAN\t",
 'nan ' => "NAN "
);
my %test_rdb_words = 
(
# ** CSCRDB files have different HISTORY line record format.. **
$version => "ASCDS_VERSION",
'\d\d\d\d-\d\d-\d\dT\d\d:\d\d:\d\d' => "TEST_DATE",
 '# HISTORY.*\n' => "",
 'HISTNUM.*' => "HISTNUM",
 'NaN' => "NAN",
 'nan\t' => "NAN\t",
 'nan ' => "NAN "
);

		     
my $toolpath = find_inpath($toolname, $ENV{'PATH'}, ':');
unless ( $toolpath )
{
   print $printfile "ERROR: Unable to find tool : $toolname\n";
   exit 1;
}

# Set parameters to defaults
system( "$PUNLEARN $toolname" );


dt_print_mt_header( $id, $semantic, $logfile, $printfile );

# Prepare for main loop
my $ii = 0;
my @faillist = ( );
my $time_user_total = 0;
my $time_sys_total = 0;
my $rc;
my $err_msg = "";
#########################################################################
# Loop over all tests
foreach my $testid (@testlist)
{
   # Init per-test error flag
   my $mismatch = 0;
   $err_msg = "";
   my $time_user;
   my $time_sys;

   # delete old outputs
   my $basename = sprintf "dt%.3dm%.3d", $id, $testid;
   system("$RM -f $testout/${basename}*");

   my $infile = "$testin/in${basename}.fits";
   # PROGRAMMER - put output filename here
   my $outfile = "$testout/${basename}";
   # PROGRAMMER - put output filter filename here
   my $outfile_filt = "$testout/${basename}_filt";
   # PROGRAMMER - put saved output filename here
   my $savfile = "$testsav/${basename}";
   # PROGRAMMER - put name of diff file here, if needed
   my $test_diff = "$testout/${basename}.diff";
   # PROGRAMMER - put saved output filter filename here
   my $savfile_filt = "$testout/${basename}_savfil";
   # PROGRAMMER - put test string here
   my $test_string = "";

   if ( $testid == 1 )
   {
      # copy event file to raw file
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   if ( $testid == 2 )
   {
      # copy event file to simple file
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   if ( $testid == 3 )
   {
      # copy event file to dtf file
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      # The 'cd $testout' bit is a hack to keep $testout out of the HISTORY
      # section of the output file.  Since $testout changes depending on how
      # you run the test, this isn't something we can remove with a simple
      # search and replace.
      $test_string = "cd $testout ; $DMCOPY infile=$infile outfile=${basename}.dtf kernel='TEXT/DTF'";
   }
   if ( $testid == 4 )
   {
      # copy event file to dtf file
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      # The 'cd $testout' bit is a hack to keep $testout out of the HISTORY
      # section of the output file.  Since $testout changes depending on how
      # you run the test, this isn't something we can remove with a simple
      # search and replace.
      $test_string = "cd $testout ; $DMCOPY infile=$infile outfile=${basename}.dtf kernel='TEXT/DTF-FIXED'";
   } 
  if ( $testid == 5 )
   {
      # copy from ASCII/RAW to FITS
      $infile = "$testin/in${basename}.raw";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   if ( $testid == 6 )
   {
      # copy from ASCII/SIMPLE to RAW
      $infile = "$testin/in${basename}.simple";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   if ( $testid == 7 )
   {
      # copy from ASCII/SIMPLE to SIMPLE
      $infile = "$testin/in${basename}.simple";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   if ( $testid == 8 )
   {
      # copy from ASCII/SIMPLE to DTF
      # The 'cd $testout' bit is a hack to keep $testout out of the HISTORY
      # section of the output file.  Since $testout changes depending on how
      # you run the test, this isn't something we can remove with a simple
      # search and replace.
      $infile = "$testin/in${basename}.simple";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "cd $testout ; $DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   if ( $testid == 9 )
   {
      # copy from ASCII/SIMPLE to DTF-FIXED
      # The 'cd $testout' bit is a hack to keep $testout out of the HISTORY
      # section of the output file.  Since $testout${outfile}_savfil changes depending on how
      # you run the test, this isn't something we can remove with a simple
      # search and replace.
      $infile = "$testin/in${basename}.simple";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "cd $testout ; $DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   if ( $testid == 10 )
   {
      # copy from ASCII/SIMPLE to FITS
      $infile = "$testin/in${basename}.simple";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   if ( $testid == 11 )
   {
      # copy from ASCII/DTF to RAW
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   if ( $testid == 12 )
   {
      # copy from ASCII/DTF to SIMPLE
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   if ( $testid == 13 )
   {
      # copy from ASCII/DTF to DTF
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   if ( $testid == 14 )
   {
      # copy from ASCII/DTF to DTF-FIXED
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   if ( $testid == 15 )
   {
      # copy from ASCII/DTF to FITS
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   if ( $testid == 16 )
   {
      # copy from ASCII/DTF-FIXED to RAW
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   if ( $testid == 17 )
   {
      # copy from ASCII/DTF-FIXED to SIMPLE
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   if ( $testid == 18 )
   {
      # copy from ASCII/DTF-FIXED to DTF
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   if ( $testid == 19 )
   {
      # copy from ASCII/DTF-FIXED to DTF-FIXED
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   if ( $testid == 20 )
   {
      # copy from ASCII/DTF-FIXED to FITS
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   if ( $testid == 21 )
   {
      # copy from ASCII/FITS to RAW
      $infile = "$testin/in${basename}.fits";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   if ( $testid == 22 )
   {
      # copy from ASCII/FITS to SIMPLE
      $infile = "$testin/in${basename}.fits";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   if ( $testid == 23 )
   {
      # copy from ASCII/FITS to DTF
      $infile = "$testin/in${basename}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   if ( $testid == 24 )
   {
      # copy from ASCII/FITS to DTF-FIXED
      $infile = "$testin/in${basename}.fits";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   if ( $testid == 25 )
   {
      # copy from ASCII/FITS to FITS
      $infile = "$testin/in${basename}.fits";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   if ( $testid == 26 )
   {
      # copy from ASCII/DTF with NULLSTR option to RAW with different NULLSTR
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile'[opt NULLSTR=-]' outfile=$outfile'[opt kernel=text/raw,NULLSTR=NULL]'";
   }
   if ( $testid == 27 )
   {
      # copy event file to CSCRDB file
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   if ( $testid == 28 )
   {
      # copy from ASCII/SIMPLE to CSCRDB
      $infile = "$testin/in${basename}.simple";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   if ( $testid == 29 )
   {
      # copy from ASCII/DTF to CSCRDB
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   if ( $testid == 30 )
   {
      # copy from ASCII/DTF-FIXED to CSCRDB
      $infile = "$testin/in${basename}.dtf";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   if ( $testid == 31 )
   {
      # copy from ASCII/FITS to CSCRDB
      $infile = "$testin/in${basename}.fits";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }
   if ( $testid == 32 )
   {
      # copy from ASCII/CSCRDB to RAW
      $infile = "$testin/in${basename}.cscrdb";
      $outfile = "${outfile}.raw";
      $savfile = "${savfile}.raw";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/RAW'";
   }
   if ( $testid == 33 )
   {
      # copy from ASCII/CSCRDB to SIMPLE
      $infile = "$testin/in${basename}.cscrdb";
      $outfile = "${outfile}.simple";
      $savfile = "${savfile}.simple";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/SIMPLE'";
   }
   if ( $testid ==34 )
   {
      # copy from ASCII/CSCRDB to DTF
      # The 'cd $testout' bit is a hack to keep $testout out of the HISTORY
      # section of the output file.  Since $testout changes depending on how
      # you run the test, this isn't something we can remove with a simple
      # search and replace.
      $infile = "$testin/in${basename}.cscrdb";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "cd $testout ; $DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF'";
   }
   if ( $testid == 35 )
   {
      # copy from ASCII/CSCRDB to DTF-FIXED
      # The 'cd $testout' bit is a hack to keep $testout out of the HISTORY
      # section of the output file.  Since $testout changes depending on how
      # you run the test, this isn't something we can remove with a simple
      # search and replace.
      $infile = "$testin/in${basename}.cscrdb";
      $outfile = "${outfile}.dtf";
      $savfile = "${savfile}.dtf";
      $test_string = "cd $testout ; $DMCOPY infile=$infile outfile=$outfile kernel='TEXT/DTF-FIXED'";
   }
   if ( $testid == 36 )
   {
      # copy from ASCII/CSCRDB to FITS
      $infile = "$testin/in${basename}.cscrdb";
      $outfile = "${outfile}.fits";
      $savfile = "${savfile}.fits";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='FITS'";
   }
   if ( $testid == 37 )
   {
      # copy from ASCII/CSCRDB to CSCRDB
      $infile = "$testin/in${basename}.cscrdb";
      $outfile = "${outfile}.cscrdb";
      $savfile = "${savfile}.cscrdb";
      $test_string = "$DMCOPY infile=$infile outfile=$outfile kernel='TEXT/CSCRDB'";
   }

   # run the tool
   $test_string .= " >> $logfname 2>&1";
   dt_print_test_header( $testid, $test_string, $logfile, $printfile );
   close $logfile;
   ($rc, $time_user, $time_sys) = dt_run_test($test_string);
   open $logfile, ">>$logfname";
   select($logfile); $| = 1; select($printfile);
   unless ( $rc == 0 )
   {
      $err_msg = "ERROR: $toolname failed to run\n";
   }
   
   my $block_typesref = $test_blocks{$testid};
   my %block_types = %$block_typesref;
   if ( $block_types{"table"} )
   {
      my @blocks = @{$block_types{"table"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmdiff_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"image"} )
   {
      my @blocks = @{$block_types{"image"}};
      foreach my $block ( @blocks )
      {
	 $err_msg .= dmimgcalc_test( $outfile, $savfile, $block, "$testin/tolerance", $logfname, $test_diff );
      }
   }
   if ( $block_types{"list"} )
   {
      $err_msg .= dmlist_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \@tolkeys, $logfname, $test_diff, "" );
   }
   if ( $block_types{"text"} )
   {
      $err_msg .= diff_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \%test_words, $logfname, $test_diff );
   }
   if ( $block_types{"text2"} )
   {
      $err_msg .= diff_test( $outfile, $outfile_filt, $savfile, $savfile_filt, \%test_rdb_words, $logfname, $test_diff );
   }

   unless ( $err_msg eq "" )
   {   
      push( @faillist, $testid );
      $mismatch = 1;
   }
   $time_user_total += $time_user;
   $time_sys_total  += $time_sys;
   dt_print_test_footer( $testid, $mismatch, $err_msg, $time_user, $time_sys, $logfile, $printfile );

   $ii++;
}

if ( scalar @faillist > 0 )
{
   $rc = 1;
}
else
{
   $rc = 0;
}
$err_msg = "@faillist";
dt_print_mt_footer( $id, $semantic, $rc, $err_msg, $time_user_total, $time_sys_total, $logfile, $printfile );

close $logfile;
close $printfile;
exit $rc;
