# Support python 2 and 3
from __future__ import print_function

import unittest
import sys,os,re
from pydoc import help   # for run-time help
from caldb4 import *     # the PACKAGE

import subprocess  


def show_whoami():
    print ('\n---', sys._getframe(1).f_code.co_name)
    sys.stdout.flush()

def usage():
    """Show the usage with example for Caldb python module."""
    
    print ("""\nUsage:\n\
           % python test_caldb4.py [opt] [-v]
           opt:
             -t1:   test the normal procedure
             -t2:   test the ways of get 'allfile'
             -t3:   test resetting 'product' and moving on
             -t4:   test setting telescope and instrument
             -t5:   test setting unrelevant attributes
             -t6:   test updating 'product'
             -t7:   test updating 'infile' option
              -u:   this message\n
            -v:   debug option\n""")

    sys.stdout.flush()

def _split_path_and_file(afile, delim):
    p,f=0,0
    delim.strip()
    p,f=re.split(delim, afile, flags=re.IGNORECASE)            
    f.strip()  # remove white space
    return p,f
    

 
DEBUG=0
class TestCaldb4lib(unittest.TestCase):
    def setUp(self):
        global DEBUG
        if "-v" in  sys.argv:
            DEBUG=1
            #sys.argv.remove("-v")
            
    def tearDown(self):
        """ un-import caldb4 ?"""
        nm=['caldb4', '']
        while True:
            if nm[0] in sys.modules:
                del (sys.modules[nm[0]])
                del (nm[0])
            else:
                break
        return
#=============================================================================
#
#   --- The Tester ---
#
#=============================================================================

#=======================================
# test 1 - test the normal procedure
#========================================
    def test1(self):
        if DEBUG: show_whoami()
        c = Caldb("chandra", "acis", "badpix")
        if DEBUG: print (c)
  
        #c.start="2000-01-29T20:00:00"
        c.start="2000-11-28"

        c.fp_temp=158.0,"K"
  
        c.grating ='None'
        if DEBUG: print (c)

        files=c.search
        num=0
        if files != None:
            print ("\nnum files: ",  len(files))
            ii=1
            for f in files:
                print (ii,"> ", os.path.basename(f) )
                ii += 1

        c.close


#=======================================
# test 2 - test the ways of get 'calfile'
#========================================
    def test2(self):
        if DEBUG: show_whoami()
        c = Caldb("chandra", "acis")
        if DEBUG: print (c)
  
        c.product="badpix"
        #c.start="2000-01-29T20:00:00"
        c.start="2000-11-28"
        if DEBUG: print (c)

        myfiles=c.search
        num=0
        if myfiles != None:
            num=len(myfiles)
        print ("\nnum files: ", num)
        for ii in range(1,num+1):
            print (ii,"> ", os.path.basename(myfiles[ii-1]))

        c.close
#=======================================
# test 3 - reset product and moving on
#========================================
    def test3(self):
        if DEBUG: show_whoami()
        c = Caldb("chandra", "acis", "badpix")
        if DEBUG: print (c)


        #c.start="2000-01-29T20:00:00"
        c.start="2000-11-28"
        if DEBUG: print (c)

        files=c.search
        num=len(files)
        print ("num files: ", num)
        for ii in range(1,num+1):
            print (ii,"> ", os.path.basename(files[ii-1]))

        c.product="det_gain"
        if DEBUG: print (c)

        files=c.search
        num=len(files)
        print ("num files: ", num)
        for ii in range(1,num+1):
            print (ii,"> ", os.path.basename(files[ii-1]))


        c.close

#======================================================
# test 4 - telescope and instrument are not updatable
#======================================================
    def test4(self):
        if DEBUG: show_whoami()
        c = Caldb("chandra", "acis","det_gain")
        if DEBUG:
            print ("My curr inputs/needs: ", c)
            print ("\nset 'telescope' and 'instrument' with new values")
        c.telescope="newteless"
        c.instrume="newinstrr"
        if DEBUG: print ("My curr inputs/needs: ", c)
        files=c.search
        num=len(files)
        print ("num files: ", num)
        for ii in range(1,num+1):
            print (ii,"> ", os.path.basename(files[ii-1]))


        if DEBUG:
            print ("\nset 'tele' and 'inst' with new values")
            c.tele="newtele"
            c.inst="newinst"
            print ("My curr inputs/needs: ", c)

        c.close

#=======================================
# test 5 - set unrelevant attributes
#========================================
    def test5(self):
        if DEBUG: show_whoami()
        c = caldb4.Caldb("chandra", "acis")
        if DEBUG: print ( c)

        c.product="badpix"
        #c.start="2000-01-29T20:00:00"
        c.start="2000-11-28"
        c.unknown = 'gabage'
        if DEBUG: print (c)

        files=c.search
        num=len(files)
        if num >=3:
            num=3

        print ("num files: ", num)
        for ii in range(1,num+1):
            print (ii,"> ", os.path.basename( files[ii-1]))

        c.close
#======================================================
# test 6 - sync test of the updated product
#======================================================
    def test6(self):
        if DEBUG: show_whoami()
        c = Caldb("chandra", "acis","t_gain")
        if DEBUG: print ("My curr inputs/needs:\n", c)
        files=c.search
        nfiles=len(files)
        print ("found ", nfiles, " files, the 1st file:")
        print (os.path.basename(files[0]), "\n")

        c.product="bkgrnd"
        if DEBUG:
            print ("Updating product to ", repr(c.product))
            print ("My curr inputs/needs:\n", c)
        files=c.search
        nfiles=len(files)
        if DEBUG:
            print ("expected ", nfiles, " files found for the inputs above.")

        c.CTI_APP='NNNNNNNNNN'
        c.SIM_Z= '-233.5874344608','mm' #-227.7139939164
        c.fp_temp='148.2-158.2','K'
        c.CCD_ID=3
        if DEBUG: print ("My curr inputs/needs:\n", c)
        cfiles=c.search
        nn=len(files)
        if DEBUG: print ("found ", nn, " files:")
        exp1= 'acis3iD2000-01-29bkgrndN0003.fits'
        exp2='acis3iD2000-12-01bkgrndN0002.fits'

        for f in files:
            if f.find(exp1) != -1 or f.find(exp2) != -1:
                print ("expected: ",os.path.basename(f))
        if DEBUG: print ("-------------\n")

        del c.sim_z
        del c.fp_temp
        del c.cti_app
        del c.ccd_id
        if DEBUG:
            print ("Delete sim_z, fp_temp, cti_app, and ccd_id.")
            print ("My curr inputs/needs: ", c)
        files = c.search
        nfiles=len(files)
        if nfiles == nn:
            print ("expected: ", nfiles, " files found.")
        else:
            print ("unexpected: ", nn, " files found compared to ",nfiles,".")
        if DEBUG: print ("-------------\n")

        c.close

#======================================================
# test 7 - test 'infile' auto-querying
#======================================================
    def test7(self):
        if DEBUG: show_whoami()

        thepath = os.getenv("TESTIN")
        if thepath==None:
            print ("\nTESTIN is not environ. variable. skip the test.")
            return
        thepath += "/calquiz"
        file="obs_3399_evt1.fits"
        infile= thepath +'/'+file
        if not os.path.exists(infile):
             #skip the testing
            print ("\ninfile '",infile,"', does not exists. Skip the test.")
            return
           

        c = Caldb("","","GREFF", infile)
        c.grating='letg'
        c.shell='0100'

        files=c.search
      
        nfiles=0
        if not files:
            print ("Error: un-expected: ", nfiles, " files found via infile-auto-querying.")
            return

        nfiles=len(files)
        nn   = 1          # expected
        exp1 = "letgD1996-11-01greffpr001N0005.fits"
        if nfiles==nn and files[0].find(exp1) != -1:
            print ("expected: ", nfiles, " files found via infile-auto-querying.")
        else:
            if nfiles!=nn:    
                print ("unexpected: ", nfiles, " files found compared to ",nn,".")
            else:
                print ("unexpected: '", file[0], "' found compared to '",exp1,"'.")

#===========================================================
# test 8 - test default arguments: tele, inst, prod,infile
#=========================================================
    def test8(self):

        c = Caldb(None)            # exit error
        c = Caldb("")              # exit error
        c = Caldb(0)               # exit error

        c = Caldb(instrume=None)   # True == instrueme="", and others to defaults
        c.close

        c = Caldb(infile=None)     # True == infile="",    and others to defaults
        c.close

        c = Caldb()                # all to defaults
        c.close


#===========================================================
# test 9 - test CALDB environment variable not set
#=========================================================
    def test9(self):


        caldbdir = os.environ['CALDB']
        os.environ['CALDB'] = ""

 
        #       try:
        #proc = subprocess.Popen(["python", "-c", "import caldb4; caldb4.Caldb()"], stdout=subprocess.PIPE)
        #out = proc.communicate()[0]
        #print(out)
        #           outs = caldb4.convert_to_ascii(out)
        #           for o in outs:
        #               print(o)
        #               raise caldb4.caldbError(out)


        #  telescope, instrume,  product
        try:
            c = Caldb("chandra",  None, "msidmap")
            c.close
        except caldb4.caldbError as e:
            print("pycaldb4# "+e.value) 
        finally:
            # restore CALDB setting
            os.environ['CALDB'] = caldbdir
  

        try:
            c=Caldb(instrume="hrc", product="tgmask2")
            myfiles=c.search
            f=myfiles
            for afile in myfiles:
                f=afile
                if afile.lower().find("caldb4/")  !=-1:
                    p,f = _split_path_and_file(afile,"caldb4/")
                elif afile.lower().find("caldb/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb/")

            c.close
            print( "<CALDB>/"+f)  # expect <CALDB>/data/chandra/hrc/tgmask2/letgD1999-07-22regN0002.fits[1]


            c=Caldb(instrume="",product="tgmask2")
            myfiles=c.search
            f=0
            for afile in myfiles:
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    # do nothing now
                    f=afile
            print( "<CALDB>/"+f)  # expect <CALDB>/data/chandra/hrc/tgmask2/letgD1999-07-22regN0002.fits[1]
     
            c=Caldb(instrume=None,product="tgmask2")
            myfiles=c.search
            for afile in myfiles:
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    # do nothing now
                    f=afile
            print( "<CALDB>/"+f)  # expect <CALDB>/data/chandra/hrc/tgmask2/letgD1999-07-22regN0002.fits[1]
 
            c=Caldb(instrume=0,product="tgmask2")
            myfiles=c.search
            for afile in myfiles:
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    # do nothing now
                    f=afile
            print( "<CALDB>/"+f)  # expect <CALDB>/data/chandra/hrc/tgmask2/letgD1999-07-22regN0002.fits[1]
 
            c=Caldb(product="tgmask2", instrume=None)
            myfiles=c.search
            f=0
            for afile in myfiles:
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    # do nothing now
                    f=afile
            print( "<CALDB>/"+f)  # expect <CALDB>/data/chandra/hrc/tgmask2/letgD1999-07-22regN0002.fits[1]
 

            # expected nothing return as 'instrume=default'
            c=Caldb(product="tgmask2")
            myfiles=c.search
            f="[]"
            for afile in myfiles:
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    # do nothing now
                    f=afile
            print( "<CALDB>/"+f)  # expect <CALDB>/[]
 


            c=Caldb()
            myfiles=c.search  # expected error

        except caldb4.caldbError as e:
            print( e.value )

#===========================================================
# test 10 - test CALDB string types
#=========================================================
    def test10(self):

        try:
         #1 
            print("\n-- Testing all types of strings")
            c=Caldb( b"chandra", u"acis", None, "")
            print('tele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, ', infile=', c.infile)
            print(c.search)
            print(c.errmsg)
            c.close
            # expect:
            # tele= chandra , instru= acis , prod=  , infile=
            # []
            # Error: not set product yet.

        #2 
            print("\n-- Testing inputs with defaults input")
            c=Caldb() 
            c.product="badpix"
            print('tele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            print(c.search)
            print(c.errmsg)
            c.close
            # expect:  ('chandra', 'default', 'badpix', '') 
            #          []
 

         #3 
            print("\n-- Testing product='' ")
            c=Caldb( "chandra", "acis", b"")
            print('tele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            print(c.search)
            print(c.errmsg)
            c.close
            # expect: Error: not set product yet

        except caldb4.caldbError as e:
            print( e.value )
    
#===========================================================
# test 11 - test c-CALDB-lib crash fixing
#=========================================================
    def test11(self):

        try:

            # start new queries with caldb-initiation
            c=Caldb(instrume=b"", product="badpix")
            print('tele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            print(c.allneeded)
            myfiles=c.search
            if myfiles:
                afile=myfiles[0]
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    f=afile
                print("#founds: ", len(myfiles), " 1st file: ",f)

            # start new queries under the same caldb-initiation
            c.detnam= b"hrc-i"
            c.fp_temp=158, b"k"
            myfiles=c.search            
            print('\ntele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",(c.detnam).decode())
            print(c.allneeded)
            if myfiles:
                afile=myfiles[0]
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    f=afile
                print("#founds: ", len(myfiles), " 1st file: ",f)
                       
  
            # start the 3rd new queries under the same caldb-initiation
            del c.fp_temp
            c.detnam=u"acis"
            myfiles=c.search
            print('\ntele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            if myfiles:
                afile=myfiles[0]
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    f=afile
                print("#founds: ", len(myfiles), " 1st file: ",f)


            # start the 4th new queries under the same caldb-initiation
            c.detnam="hrc-s"
            myfiles=c.search
            print('\ntele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            if myfiles: 
                afile=myfiles[0]
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    f=afile
                print("#founds: ", len(myfiles), " 1st file: ",f)


            # start the 5th new queries under the same caldb-initiation
            # test set-instrume is no-effect on the search, as the 'instrume' is  retained as whatever in Caldb() initiation 
            #  set-instrume with 'acis'
            del c.detnam
            c.instrume="acis"
            myfiles=c.search
            print('\ntele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            if len(myfiles):
                afile=myfiles[0]
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    f=afile
                print("#founds: ", len(myfiles), " 1st file: ",f)

            # start the 6th new queries under the same caldb-initiation
            # set-instrume with 'default'
            c.instrume="default" 
            myfiles=c.search
            print('\ntele=',c.telescope, ', instru=', c.instrume, ', prod=', c.product, " detnam=",c.detnam)
            if len(myfiles):
                afile=myfiles[0]
                if afile.lower().find("caldb/") !=-1:
                    p,f = _split_path_and_file(afile,"caldb/")
                elif afile.lower().find("caldb4/") !=-1:
                    p,f=_split_path_and_file(afile, "caldb4/")
                else:
                    f=afile
                print("#founds: ", len(myfiles), " 1st file: ",f)
        except caldb4.caldbError as e:
            print( e.value )


        c.close
 

if __name__ == "__main__":

    if "-u" in sys.argv[1:]:
        print (usage() )     
    elif len(sys.argv)==1 or (len(sys.argv)==2 and ("-v" in sys.argv or "-q" in sys.argv ) ):
        unittest.main(defaultTest="TestCaldb4lib")
    else:
        suite = unittest.TestSuite()
        for arg in sys.argv[1:]:
            if arg == "-t1":
                suite.addTest(TestCaldb4lib("test1")) 
                sys.argv.remove("-t1")
            elif arg == "-t2":
                suite.addTest(TestCaldb4lib("test2"))
                sys.argv.remove("-t2")
            elif arg == "-t3":
                suite.addTest(TestCaldb4lib("test3"))
                sys.argv.remove("-t3")
            elif arg == "-t4":
                suite.addTest(TestCaldb4lib("test4"))
                sys.argv.remove("-t4")
            elif arg == "-t5":
                suite.addTest(TestCaldb4lib("test5"))
                sys.argv.remove("-t5")
            elif arg == "-t6":
                suite.addTest(TestCaldb4lib("test6"))
                sys.argv.remove("-t6")
            elif arg == "-t7":
                thepath=os.getenv("TESTIN")
                if thepath != None:
                    suite.addTest(TestCaldb4lib("test7"))
                    sys.argv.remove("-t7")
            elif arg == "-t8":
                suite.addTest(TestCaldb4lib("test8"))
                sys.argv.remove("-t8")
            elif arg == "-t9":
                suite.addTest(TestCaldb4lib("test9"))
                sys.argv.remove("-t9")
            elif arg == "-t10":
                suite.addTest(TestCaldb4lib("test10"))
                sys.argv.remove("-t10")
            elif arg == "-t11":
                suite.addTest(TestCaldb4lib("test11"))
                sys.argv.remove("-t11")
            else:
                pass

        unittest.TextTestRunner().run(suite)
     
