#!/usr/bin/perl
#

# Make sure INC path includes test_utils and such
@lib_paths = split /\s+/, $ENV{'ASCDS_INSTALL'};
$ASCDS_INSTALL = shift @lib_paths;
if ( -d "./datamodel" )
{
   $TEXEC_DIR = "./datamodel";
}
elsif ( -d "$ASCDS_INSTALL/test/bin/datamodel" )
{
   $TEXEC_DIR = "$ASCDS_INSTALL/test/bin/datamodel";
}
else
{
   print "ERROR: Can't find datamodel test scripts.";
   exit 1;
}

unshift(@INC, $TEXEC_DIR);

require 'test_utils.pl';
$RM = find_inpath( "rm", $ENV{'PATH'}, ':' );
$MKDIR = find_inpath( "mkdir", $ENV{'PATH'}, ':' );


# Use perl's strict mode
use strict;
use vars '$RM', '$MKDIR', '$TEXEC_DIR', '$ASCDS_INSTALL';



# Turn off buffering
select(STDOUT); $| = 1;
#
# Set some variables used by 'sort'
$ENV{'LC_COLLATE'}  = "en_US.ISO8859-1";
$ENV{'LC_MESSAGES'} = "C";


#
# Process Date
my $date = `date +%d%b%Y_%H:%M:%S`;
chop($date);

my ($junk,$start_hours,$start_min,$start_sec ) = $date =~ /(.*_)(\d*):(\d*):(\d*)/;

# Define local variables from environment
my $USER    = $ENV{'USER'};
my $UPARM   = $ENV{'ASCDS_USER_PARAM'};
my $version = $ENV{'ASCDS_VERSION'};
my $home    = $ENV{'HOME'};
my $thisdir = $ENV{'PWD'};

my $testin  = "$ENV{'TESTIN'}/datamodel";
my $testout = "$ENV{'TESTOUT'}/datamodel";
my $testlog = "$ENV{'TESTLOG'}/datamodel";
my $platform = `uname`;
chop $platform;
my $testsav;
if ( $ENV{'TESTSAV'} )
{
  $testsav = "$ENV{'TESTSAV'}/datamodel";
}
else
{
  $testsav = "$ASCDS_INSTALL/test/baseline/datamodel";
}

my $testrun = $ENV{'TESTRUN'};
my $caldb   = $ENV{'CALDB'};
my $printfile = *STDOUT;


#
#------------------------------------------------------------
# Load Arguments

my %inargs;
%inargs = &parse_args(\@ARGV);



#
# Process Arguments


# ***********
# -h = Help
# ***********
if ( exists $inargs{"-h"} )
{
   print "Usage: \n";
   print "   testdm [ -ahliosfprtm ] \n";
   print "      -h   Help ( this text ) \n";
   print "\n";
   print "      -a   Run all DM tests \n";
   print "\n";
   print "      -l   List all DM tests \n";
   print "\n";
   print "      -o   Specify test output directory \n";
   print "\n";
   print "      -i   Specify test input directory \n";
   print "\n";
   print "      -s   Specify test saved output directory \n";
   print "\n";
   print "      -f   Specify log file \n";
   print "\n";
   print "      -p   Specify printed output file \n";
   print "\n";
   print "      -r   Remove output from tests that pass \n";
   print "\n";
   print "      -t   Specify comma separated list of DM mini-test ids \n";
   print "\n";
   print "      -m   Specify comma separated list of DM mini-test semantics \n";
   print "\n";
   print "      -g   Specify comma separated list of DM mini-test groups \n";
   print "\n";
   print "      -W   Specify location of output HTML report.\n";
   print "\n\n";
   exit (0);
}

#
# Load Test List
my $testdir = "$TEXEC_DIR";
my %testlist;
%testlist = load_test_list($testdir);
if ( exists $inargs{'-l'} )
{
   print "ID\tSEMANTIC\n";
   print "---\t--------\n";
   my @arr = sort keys %testlist;
   foreach my $dm_test ( @arr )
   {
      my $test_script = $testlist{$dm_test};
      my ($id, $semantic) = $test_script =~ /^dt(\d{3})([\w_]*)/;
      print "$id\t$semantic\n";
   }
   exit (0);
}


my $htmlfile = "";
if ( $inargs{'-W'} )
{
   $htmlfile = $inargs{'-W'};
}
my $extra = "-k FITS";
if ( $inargs{'-i'} )
{
   $testin = $inargs{'-i'};
   $extra .= " -i $testin";
}
if ( $inargs{'-o'} )
{
   $testout = $inargs{'-o'};
   $extra .= " -p $testout";
}
if ( $inargs{'-s'} )
{
   $testsav = $inargs{'-s'};
   $extra .= " -s $testsav";
}
unless ( -d $testlog ) { system( "$MKDIR -p $testlog" ); }
system( "$RM -f $testlog/datamodel.log.*" );
system( "$RM -f $testlog/dt*" );
unless ( -d $testout ) { system( "$MKDIR -p $testout" ); }
system( "$RM -f $testout/dt*" );
unless ( -d $testin )
{
   print $printfile "ERROR: can't find input directory : $testin\n";
   exit 1;
}
unless ( -d $testsav )
{
   print $printfile "ERROR: can't find saved output directory : $testsav\n";
   exit 1;
}

# open log files
if ( $inargs{'-p'} )
{
   unless ( open TESTPRINT, ">>$inargs{'-p'}" )
   {
      print STDOUT "ERROR: Failed to open verbose output file : $inargs{'-p'}\n";
      exit 1;
   }
   $printfile = *TESTPRINT;
}
my $logfile;
my $logfname = "$testlog/datamodel.log.$date";
if ( $inargs{'-f'} )
{
   $logfname = $inargs{'-f'};
}
if ( open TESTLOG, ">>$logfname" )
{
   $logfile = *TESTLOG;
}
else
{
   print $printfile "ERROR: Failed to open log file : $logfname\n";
   exit 1;
}


if ( $inargs{'-t'} )
{
   my @arr = split( ",", $inargs{'-t'} );
   my %test_ids = ( );
   foreach my $tmp ( @arr )
   {
      $test_ids{$tmp} = 1;
   }
   foreach my $dm_test ( keys %testlist )
   {
      if ( ! exists $test_ids{$dm_test} )
      {
	 delete $testlist{$dm_test};
      }
   }
}
my %test_semantics = ( );
if ( $inargs{'-m'} )
{
   my @arr = split( ",", $inargs{'-m'} );
   foreach my $tmp ( @arr )
   {
      $test_semantics{$tmp} = 1;
   }
   foreach my $dm_test ( keys %testlist )
   {
      my $test_script = $testlist{$dm_test};
      my ($id, $semantic) = $test_script =~ /^dt(\d{3})([\w_]*)/;
      if ( ! exists $test_semantics{$semantic} )
      {
	 delete $testlist{$dm_test};
      }
   }
}

my $remove_pass_output = 0;
if ( exists $inargs{'-r'} )
{
   $remove_pass_output = 1;
}

print $logfile "Running DataModel Tests $date\n";
print $logfile 
"========================================================================\n\n";
print $logfile "
Environment
-----------
TEST INPUT DIR  : $testin
TEST OUTPUT DIR : $testout
TEST SAVED DIR  : $testsav
TEST LOG DIR    : $testlog
ASCDS VERSION   : $version
PLATFORM        : $platform

";
print $printfile "Running DataModel Tests $date\n";
print $printfile 
"========================================================================\n\n";
print $printfile "Logfile at: $logfname\n\n";
my $time_user_total = 0;
my $time_sys_total = 0;
my $time_user;
my $time_sys;
my @sys_times = ( );
my @user_times = ( );
my ( $passed_total, $failed_total ) = (0,0);
my $rc;
my @test_keys = sort(keys %testlist);
foreach my $dm_test ( @test_keys )
{
   my $test_script = $testlist{$dm_test};
   my ($id, $semantic) = $test_script =~ /^dt(\d{3})([\w_]*)/;
   my $basename = sprintf( "dt%.3d${semantic}", $dm_test );
   my $minilog = "$testlog/$basename.log.$date";
   print $printfile "Running test $dm_test ($semantic)\n";
   print $printfile "Log at $minilog\n";
   print $logfile "Running test $dm_test ($semantic)\n";
   print $logfile "Log at $minilog\n";
   close $logfile;
   my $comstr = "$testdir/$test_script $extra -f $minilog >> $logfname 2>&1";
   ($rc, $time_user, $time_sys) = dt_run_test($comstr);
   push( @user_times, $time_user );
   push( @sys_times, $time_sys );
   open $logfile, ">>$logfname";

   if ( $rc == 0 )
   {
      print $printfile "PASSED\n";
      print $logfile "PASSED\n";
      $passed_total++;

      if ( $remove_pass_output )
      {
	 $comstr = sprintf( "$RM -f $testout/dt%.3dm*", $id );
	 system( $comstr );
      }
   }
   else
   {
      print $printfile "FAILED\n";
      print $logfile "FAILED\n";
      $failed_total++;
   }
   print $logfile "CPU TIME: USER: $time_user s. SYS: $time_sys s.\n\n";
   print $printfile "CPU TIME: USER: $time_user s. SYS: $time_sys s.\n\n";
   $time_user_total += $time_user;
   $time_sys_total += $time_sys;
}

print $logfile "
=============================================================================
FINISHED DATAMODEL TESTS
PASSED: $passed_total
FAILED: $failed_total
TOTAL CPU TIME: USER: $time_user_total s. SYS: $time_sys_total s.

";
print $printfile "
=============================================================================
FINISHED DATAMODEL TESTS
PASSED: $passed_total
FAILED: $failed_total
TOTAL CPU TIME: USER: $time_user_total s. SYS: $time_sys_total s.

";

close $logfile;
if ( $htmlfile ne "" )
{
   dt_gen_web_rpt( $logfname, $htmlfile );
}

sub load_test_list {
   my ( $testdir ) = @_;
   my %testlist;

   opendir TDIR, "$testdir" or return \%testlist;
   my @files = readdir TDIR;
   my $file_count = 0;
   foreach my $file ( @files ) 
   {
      if ( $file =~ /^dt\d{3}[a-zA-Z_0-9.]*$/ )
      {
	 my ( $id ) = ( $file =~ /^dt(\d{3})/ );
	 $testlist{$id} = $file;
      }
   }
   return %testlist;
}
